const { app, BrowserWindow, dialog, ipcMain } = require('electron');
const { exec } = require('child_process');
const axios = require('axios');
const fetch = require("node-fetch");
const fs = require('fs');
const os = require('os');
const settings = require('electron-settings');

var win = null;

function createWindow() {
  win = new BrowserWindow({
    width: 450,
    height: 600,
    webPreferences: {
      contextIsolation: false,
      enableRemoteModule: true,
      nodeIntegration: true
    }
  })

  win.removeMenu();
  //win.webContents.openDevTools();
  win.loadFile('src/index.html');
}

function prinTesTicket(printerName) {
  var esc = '\x1B'; //ESC byte in hex notation
  var newLine = '\x0A'; //LF byte in hex notation
  var cmds = esc + "@"; //Initializes the printer (ESC @)
  cmds += esc + '!' + '\x38'; //Emphasized + Double-height + Double-width mode selected (ESC ! (8 + 16 + 32)) 56 dec => 38 hex
  cmds += 'PRUEBA'; //text to print
  cmds += newLine + newLine;
  cmds += esc + '!' + '\x00'; //Character font A selected (ESC ! 0)
  cmds += 'COOKIES                   5.00';
  cmds += newLine;
  cmds += 'MILK 65 Fl oz             3.78';
  cmds += newLine + newLine;
  cmds += 'SUBTOTAL                  8.78';
  cmds += newLine;
  cmds += 'TAX 5%                    0.44';
  cmds += newLine;
  cmds += 'TOTAL                     9.22';
  cmds += newLine;
  cmds += 'CASH TEND                10.00';
  cmds += newLine;
  cmds += 'CASH DUE                  0.78';
  cmds += newLine + newLine + newLine + newLine;
  cmds += newLine + newLine + newLine + newLine;

  // add cut command
  var cutComm = settings.getSync('printer.cut').split('.');
  if (cutComm.length > 0) {
    cmds += String.fromCharCode(...cutComm) + newLine;
  }

  // add open command
  var openComm = settings.getSync('printer.open').split('.');
  if (openComm.length > 0) {
    cmds += String.fromCharCode(...openComm) + newLine;
  }

  fs.writeFile("ticket", cmds, function (err) {
    if (err) {
      return console.log(err);
    }

    prinTicket(printerName);
  });

  // volcamos en consola la ruta del archivo
  console.log(app.getAppPath() + "/ticket");
}

function prinTicket(printerName) {
  var printCmd = 'lp -d "' + printerName + '" ticket';
  if (os.platform() == 'win32') {
    var fullPath = '"' + app.getAppPath() + "\\";
    printCmd = fullPath + 'RawPrint.exe" "' + printerName + '" ticket';
  }

  exec(printCmd, (error, stdout, stderr) => {
    if(stdout) {
      console.log(`stdout: ${stdout}`);
    }
    if (error) {
      console.log(`error: ${error.message}`);
      dialog.showErrorBox("print error", error.message);
      return;
    }
    if (stderr) {
      console.log(`stderr: ${stderr}`);
      return;
    }
  });
}

app.whenReady().then(createWindow)

app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
})

app.on('activate', () => {
  if (BrowserWindow.getAllWindows().length === 0) {
    createWindow();
  }
})

ipcMain.on('print-test', (event, arg) => {
  prinTesTicket(arg);
  event.reply('print-test', 'ok');
})

ipcMain.on('send-to-printer', (event, arg) => {
  console.log('send-to-printer');

  var text = '\x1B' + '@' + arg;
  fs.writeFile("ticket", text, function (err) {
    if (err) {
      return console.log(err);
    }

    prinTicket(settings.getSync('printer.name'));
  });

  event.reply('send-to-printer', 'ok');
})

// http server
const http = require('http');
const requestListener = function (req, res) {
  res.setHeader('Access-Control-Allow-Origin', '*');
  res.writeHead(200);
  res.end('mc20printer 1.1.0');
  console.log('http request: ' + req.method + ' ' + req.url);

  // send a message to the main window
  if (req.method == 'GET') {
    win.webContents.send('http-request', req.url);
  }
}
const server = http.createServer(requestListener);
server.listen(8089).on('error', (err) => {
  if (err) {
    console.log(err);
    app.quit();
  }
});