const { ipcRenderer, remote } = require("electron");
const axios = require('axios');
const printers = remote.getCurrentWebContents().getPrinters();
const settings = require('electron-settings');

async function connectionPing() {
  var apiUrl = document.getElementById("api_url").value;
  var apiKey = document.getElementById("api_key").value;

  try {
    const apiClient = await axios.create({ baseURL: apiUrl, headers: { Token: apiKey } });
    printPendingTickets(apiClient);
  } catch (err) {
    alert('La conexión falló: ' + err);
  }
}

async function printPendingTickets(apiClient) {
  try {
    // get the latest tickets
    const resTickets = await apiClient.get('/api/3/ticketes?filter[printed]=0');
    console.log(resTickets);

    // find the apikey in resPrinters and update lastactivity
    for (const item of resTickets.data) {
      if (item.appversion > 1.0) {
        alert('Por favor, descarga la nueva versión de la app');
        // open a browser window with the app url https://megacity20.com/support/59
        remote.shell.openExternal('https://megacity20.com/support/59');
      } else if (item.base64) {
        ipcRenderer.send('send-to-printer', atob(item.body));
      } else {
        ipcRenderer.send('send-to-printer', item.body);
      }

      // set printed to true
      const resPut = await apiClient.put('/api/3/ticketes/' + item.id, 'printed=1');
    }

    // get the printer list
    const resPrinters = await apiClient.get('/api/3/ticketprinteres');

    // find the apikey in resPrinters and update lastactivity, len, cut and open
    for (const item of resPrinters.data) {
      if (item.apikey == document.getElementById("api_key").value) {
        // get the current date YYYY-MM-DD HH:MM:SS format
        var date = new Date();
        var dateStr = date.getFullYear() + '-' + (date.getMonth() + 1) + '-' + date.getDate()
          + ' ' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();

        // send put request
        var printerLen = document.getElementById("printer_len").value;
        var printerCut = document.getElementById("printer_cut").value;
        var printerOpen = document.getElementById("printer_open").value;
        const resPut = await apiClient.put(
          '/api/3/ticketprinteres/' + item.id,
          'lastactivity=' + dateStr + '&linelen=' + printerLen + '&cutcommand=' + printerCut + '&opencommand=' + printerOpen
        );
      }
    }
  } catch (err) {
    alert('La conexión falló');
    console.log(err);
  }
}

async function testConnection() {
  var printerName = document.getElementById("printer_list").value;
  var apiUrl = document.getElementById("api_url").value;
  var apiKey = document.getElementById("api_key").value;
  var printerLen = document.getElementById("printer_len").value;
  var printerCut = document.getElementById("printer_cut").value;
  var printerOpen = document.getElementById("printer_open").value;

  if (apiUrl.substring(0, 7) != 'http://' && apiUrl.substring(0, 8) != 'https://') {
    alert('URL incorrecta');
    return false;
  } else if (0 == apiKey.length) {
    alert('Se necesita una API key');
    return false;
  }

  try {
    const apiClient = await axios.create({ baseURL: apiUrl, headers: { Token: apiKey } });
    const resPrinters = await apiClient.get('/api/3/ticketprinteres');
    console.log(resPrinters);

    // find the apikey in resPrinters and update lastactivity, len, cut and open
    for (const item of resPrinters.data) {
      if (item.apikey == apiKey) {
        // get the current date YYYY-MM-DD HH:MM:SS format
        var date = new Date();
        var dateStr = date.getFullYear() + '-' + (date.getMonth() + 1) + '-'
          + date.getDate() + ' ' + date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();

        // send put request
        const resPut = await apiClient.put(
          '/api/3/ticketprinteres/' + item.id,
          'lastactivity=' + dateStr + '&linelen=' + printerLen + '&cutcommand=' + printerCut + '&opencommand=' + printerOpen
        );
        console.log(resPut);

        // handle success
        settings.setSync('printer', { name: printerName, len: printerLen, cut: printerCut, open: printerOpen });
        settings.setSync('api', { url: apiUrl, key: apiKey });
        alert('La conexión funciona correctamente');
        printPendingTickets(apiClient);
      }
    }
  } catch (err) {
    alert('La conexión falló');
    console.log(err);
  }

  return false;
}

function testPrinter() {
  settings.setSync('printer', {
    name: document.getElementById("printer_list").value,
    len: document.getElementById("printer_len").value,
    cut: document.getElementById("printer_cut").value,
    open: document.getElementById("printer_open").value
  });
  ipcRenderer.send('print-test', document.getElementById("printer_list").value);
}

// load printer list
var selectedPrinter = settings.getSync('printer.name');
printers.map((item, index) => {
  if (item.name == selectedPrinter) {
    document.getElementById("printer_list").innerHTML +=
      '<option value="' + item.name + '" selected="">' + item.name + '</option>';
  } else {
    document.getElementById("printer_list").innerHTML +=
      '<option value="' + item.name + '">' + item.name + '</option>';
  }
});

// load preferences
if (settings.hasSync('api.url')) {
  document.getElementById("api_url").value = settings.getSync('api.url');
  document.getElementById("api_key").value = settings.getSync('api.key');
  document.getElementById("printer_len").value = settings.getSync('printer.len');
  document.getElementById("printer_cut").value = settings.getSync('printer.cut');
  document.getElementById("printer_open").value = settings.getSync('printer.open')
  connectionPing();
} else {
  document.getElementById("printer_len").value = '48';
  document.getElementById("printer_cut").value = '27.109';
  document.getElementById("printer_open").value = '27.112.48.55.121';
}

// hand main process message http-request
ipcRenderer.on('http-request', (event, arg) => {
  connectionPing();
});